@@ -5,7 +5,8 @@ Petal - Perl Template Attribute Language
 =head1 SYNOPSIS
 
   use Petal;
-  my $template = new Petal ( base_dir => '.', file => 'test.html' );
+  @Petal::BASE_DIR = qw |. ./templates /var/www/templates|;
+  my $template = new Petal ( 'test.xml' );
   print $template->process (
     some_hash   => $hashref,
     some_array  => $arrayref,
@@ -18,20 +19,12 @@ Join the Petal mailing list:
 
 =head1 SUMMARY
 
-Hopefully, Petal is a bit more than "yet another template engine".
+Petal is a XML based templating engine that is able to process any
+kind of XML. HTML parsing and XHTML is also supported.
 
-First of all, Petal uses XML::Parser to process XML templates and makes
-it easy to produce well-formed XML from these template pages. If your
-template is not valid XML, then Petal Petal will issue a warning and
-attempt to parse the file using HTML::TreeBuilder and generate the XML
-events using the HTML::TreeBuilder parsed tree.
-
-So stricly speaking, your templates won't *need* to be valid XML, but it
-would be good practice if they were.
-
-Besides, Petal borrows a lot from Zope Page Templates TAL specification
-in order to make template files very dreamweaver / frontpage / etc.
-friendly.
+Because Petal borrows a lot of good ideas from the Zope Page Templates
+TAL specification, it is very well suited for the creation of truly WYSIWYG
+XHTML editable templates.
 
 The idea is to enforce even further the separation of logic and
 presentation. With Petal, graphic designers who use their favorite
@@ -45,12 +38,13 @@ pages.
 =cut
 package Petal;
 use Petal::Hash;
-use Petal::Canonicalizer;
 use Petal::CodeGenerator;
 use Petal::Cache::Disk;
 use Petal::Cache::Memory;
 use Petal::Parser::XMLWrapper;
 use Petal::Parser::HTMLWrapper;
+use Petal::Canonicalizer::XML;
+use Petal::Canonicalizer::XHTML;
 use strict;
 use warnings;
 use Carp;
@@ -66,22 +60,36 @@ use vars qw /@tokens @nodeStack/;
 
 =head2 Description
 
-$PARSER - Currently acceptable values are
+$INPUT - Currently acceptable values are
 
-  'XML'  - Petal will use XML::Parser to parse the template
-  'HTML' - Petal will use HTML::TreeBuilder to parse the template
-  'ANY'  - Petal will try with XML::Parser first, then with
-           HTML::TreeBuilder if XML::Parser fails to parse the source
-           template file.
+  'XML'   - Petal will use XML::Parser to parse the template
+  'HTML'  - Petal will use HTML::TreeBuilder to parse the template
+  'XHTML' - Alias for 'HTML'
 
 This variable defaults to 'XML'.
 
 =cut
-our $PARSER = 'XML';
-our $PARSERS = {
-    'XML'  => 'Petal::Parser::XMLWrapper',
-    'HTML' => 'Petal::Parser::HTMLWrapper',
-    'ANY'  => [ 'Petal::Parser::XMLWrapper', 'Petal::Parser::HTMLWrapper' ],
+our $INPUT  = 'XML';
+our $INPUTS = {
+    'XML'   => 'Petal::Parser::XMLWrapper',
+    'HTML'  => 'Petal::Parser::HTMLWrapper',
+    'XHTML' => 'Petal::Parser::HTMLWrapper',
+};
+
+
+=pod
+
+$OUTPUT - Currently acceptable values are
+
+  'XML'   - Petal will output generic XML
+  'XHTML' - Same as XML except for tags like <br /> or <input />
+
+=cut
+our $OUTPUT  = 'XML';
+our $OUTPUTS = {
+    'XML'   => 'Petal::Canonicalizer::XML',
+    'HTML'  => 'Petal::Canonicalizer::XHTML',
+    'XHTML' => 'Petal::Canonicalizer::XHTML',
 };
 
 
@@ -121,8 +129,7 @@ $MEMORY_CACHE - If set to FALSE, Petal will not use the Petal::Disk::Memory modu
 our $MEMORY_CACHE = 1;
 
 
-# $VERSION is internal, thanks
-our $VERSION = '0.5';
+our $VERSION = '0.6';
 
 
 =head2 Example
@@ -136,8 +143,10 @@ our $VERSION = '0.5';
   $Petal::DISK_CACHE = 1;
   $Petal::MEMORY_CACHE = 1;
 
-  # let's use something horrible
-  $Petal::PARSER = 1;
+  # we are parsing a valid XHTML file, and we want to output
+  # XHTML as well...
+  $Petal::INPUT = 'XML';
+  $Petal::OUTPUT = 'XHTML';  
 
 =cut
 
@@ -179,14 +188,17 @@ Example:
 sub process
 {
     my $self = shift;
-    
+   
     # make the hash highly magical
     my $hash = (@_ == 1 and ref $_[0] eq 'HASH') ?
         new Petal::Hash (%{$_[0]}) :
         new Petal::Hash (@_);
     
     my $coderef = $self->_code_memory_cached;
-    return $coderef->($hash);
+    my $res = undef;
+    eval { $res = $coderef->($hash) };
+    if (defined $@ and $@) { confess $@ . "\n===\n\n" . $self->_code_with_line_numbers }
+    return $res;
 }
 
 
@@ -197,7 +209,7 @@ sub process
 sub _code_with_line_numbers
 {
     my $self = shift;
-    my $code = shift;
+    my $code = $self->_code_disk_cached;
     my $count = 0;
     return join "\n", map { ++$count; "$count. $_" } split /\n/, $code;
 }
@@ -315,6 +327,9 @@ sub _code_memory_cached
 	    my $cpt = Safe->new ("Petal::CPT");
 	    $cpt->reval($code_perl);
 	    die $@ if ($@);
+	    
+	    # remove silly warning '"Petal::CPT::VAR1" used only once'
+	    $Petal::CPT::VAR1 if (0);
 	    $code = $Petal::CPT::VAR1;
 	}
 	else
@@ -348,25 +363,12 @@ sub _memory_cache
 sub _canonicalize
 {
     my $self = shift;
-    my $parser_type = $PARSER;
-    my $parser_module_name = shift || $PARSERS->{$parser_type};
-    if (ref $parser_module_name)
-    {
-	foreach my $modname ( @{$parser_module_name} )
-	{
-	    my $result;
-	    eval { $result = $self->_canonicalize ($modname) };
-	    if (defined $@ and $@) { warn "BAD TEMPLATE: $@" }
-	    else                   { return $result          }
-	}
-	confess "No suitable parse module could be found";
-    }
-    else
-    {
-	my $parser = $parser_module_name->new;
-	my $data_ref = $self->_file_data_ref;
-	return Petal::Canonicalizer->process ($parser, $data_ref);
-    }
+    my $parser_type        = $INPUTS->{$INPUT}   || confess "unknown \$Petal::INPUT = $INPUT";
+    my $canonicalizer_type = $OUTPUTS->{$OUTPUT} || confess "unknown \$Petal::OUTPUT = $OUTPUT";
+    
+    my $data_ref = $self->_file_data_ref;
+    my $parser = $parser_type->new;
+    return $canonicalizer_type->process ($parser, $data_ref);
 }
 
 
@@ -375,6 +377,45 @@ sub _canonicalize
 
 __END__
 
+=head1 Overview
+
+Currently, Petal supports three different syntaxes:
+
+* A 'Canonical' Syntax
+
+  This is my variable: <?petal:var name="my_variable"?>
+
+* A 'TAL-like' Syntax
+
+  This is my variable: <span petal:replace="my_variable">Dummy Variable</span>
+
+* An 'Inline' Syntax
+
+  This is my variable: $my_variable
+
+You can use all three syntaxes in the same template file. In order to ease
+maintenance, Petal internally re-writes your templates into the canonical
+syntax.
+
+The cycle of a Petal template is the following:
+
+  1. Read the source XML template
+  2. $INPUT (XML or HTML) throws XML events from the source file
+  3. $OUTPUT (XML or HTML) uses these XML events to canonicalize the template
+  4. Petal::CodeGenerator turns the canonical template into Perl code
+  5. Petal::Cache::Disk caches the Perl code on disk
+  6. Petal turns the perl code into a subroutine
+  7. Petal::Cache::Memory caches the subroutine in memory
+  8. Petal executes the subroutine
+
+If you are under a persistent environement a la mod_perl, subsequent calls
+to the same template will be reduced to step 8 until the source template
+changes.
+
+Otherwise, subsequent calls will resume at step 6, until the source template
+changes.
+
+
 =head1 Petal Syntax Summary
 
 Petal features a flexible, XML-compliant syntax which can be summarized
@@ -657,15 +698,15 @@ Usual stuff:
     Happy Birthday, $xml:user/real_name!
   <?petal:else?>
     What?! It's not your birthday?
-    Maybe tomorrow...
+    A very merry unbirthday to you! 
   <?petal:end?>
 
 You can use petal:condition instead of petal:if, and indeed you can use
 modifiers:
 
-  <?petal:condition name=":false user/is_birthay"?>
+  <?petal:condition name="false: user/is_birthay"?>
     What?! It's not your birthday?
-    Maybe tomorrow...
+    A very merry unbirthday to you! 
   <?petal:else?>
     Happy Birthday, $xml:user/real_name!
   <?petal:end?>
@@ -709,22 +750,16 @@ it's a bug :-)
 
 Petal fully support includes using the following syntax:
 
-<?petal:include file="include.xml"?>
+  <?petal:include file="include.xml"?>
 
 And it will include the file 'include.xml', using the current object
 base_dir attribute. Petal includes occur at RUN TIME. That means that
-there is NO SUPPORT to prevent infinite includes, which is not so much
-of a deal since it happens at run time...
+there is NO SUPPORT to prevent infinite includes, which is usually not
+so much of a deal since it happens at run time...
 
 This should let you build templates which have a recursive behavior
-which can be useful to apply templates to any tree-shaped structure.
-
-Because the include happens at run time, you need to specify any Petal
-options which you might want to use in the template, i.e.
-
-<?petal:include file="include.xml" disk_cache="0" taint="1" ?>
-
-You cannot specify base_dir, which is inherited from the parent template.
+which can be useful to apply templates to any tree-shaped structure (i.e.
+sitemaps, threads, etc).
 
 If you want use XML::Parser to include files, you should make sure that
 the included files are valid XML themselves... FYI XML::Parser chokes on this:
@@ -750,9 +785,11 @@ None.
 =head1 BUGS
 
 Probably plenty at the time of this writing.
-Mail them to me and I'll squash 'em all! Begon jaune a l'attaque!
+Mail them to me and I'll squash 'em all!
+Begon jaune a l'attaque!
 
-Cache seems to fail sometimes on (Apache + Windows + mod_perl) platforms
+Problems have been reported with the petal cache on a
+(Apache + Windows + mod_perl) platform.
 
 
 =head1 AUTHOR
@@ -762,10 +799,16 @@ Copyright 2002 - Jean-Michel Hiver <jhiver@mkdoc.com>
 This module free software and is distributed under the
 same license as Perl itself.
 
-BIG thanks to William McKee <william@knowmad.com> for his useful
-suggestions, patches, and bug reports.
+Many thanks to:
 
-Thanks to Lucas Saud <lucas.marinho@uol.com.br> for the
+William McKee <william@knowmad.com> for his useful suggestions,
+patches, and bug reports.
+
+Sean M. Burke <sburke@cpan.org> for his improvements on the
+HTML::TreeBuilder module which tremendously helped with HTML
+parsing.
+
+Lucas Saud <lucas.marinho@uol.com.br> for the
 Petal::Hash::Encode_HTML he contributed.
 
 
@@ -775,17 +818,6 @@ Join the Petal mailing list:
 
   http://lists.webarch.co.uk/mailman/listinfo/petal
 
-If you want to dig Petal a bit further:
-
-  perldoc Petal::Hash
-  perldoc Petal::Hash::Var
-  perldoc Petal::Parser::XMLWrapper
-  perldoc Petal::Parser::HTMLWrapper
-  perldoc Petal::Canonicalizer
-  perldoc Petal::CodeGenerator
-  perldoc Petal::Cache::Disk
-  perldoc Petal::Cache::Memory
-
 Have a peek at the TAL / TALES / METAL specs:
 
   http://www.zope.org/Wikis/DevSite/Projects/ZPT/TAL
